#include <stdio.h>
#include <math.h>
#include <time.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "buttons.h"

void countDown(void);
void selectTime(void);
void quitApp(void);

int countdownSecs = 60; // defaults to one minute
char countdownString[256];
int width, height;
int left, top;
int running = 1;
int counting = 0;
int selectedTime = 0;



SDL_Surface *screen = NULL;
SDL_Surface *text_surface = NULL;
SDL_Surface *help1_surface = NULL;
SDL_Surface *help2_surface = NULL;
SDL_Color color = {255,255,255};
TTF_Font *font = NULL;
TTF_Font *helpFont = NULL;

int main(int argc, char *argv[]) {
	SDL_Init(SDL_INIT_VIDEO || SDL_INIT_AUDIO); // Initialize SDL
	screen = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE); // Set screen video mode 
	int ttfresult = TTF_Init(); // Initialize SDL_ttf
	SDL_ShowCursor(SDL_DISABLE); // No need for cursor on a mouse-less device
	
	if(screen == NULL) { // Did SDL load?
		printf("Couldn't set video mode:\n%s\n\nPress [START] to exit.\n", SDL_GetError());
		getchar();
		return 1;
	}
	
	if(ttfresult == -1) { // Did SDL_ttf load?
		printf("TTF_Init:\n%s\n\nPress [START] to exit.\n", TTF_GetError());
		getchar();
		return 1;
	}
	
	
	font = TTF_OpenFont("font.ttf", 72);
	helpFont = TTF_OpenFont("font.ttf", 16);
	
	if(!font) { // Did the font open?
		printf("TTF_OpenFont:\n%s\n\nPress [START] to exit.\n", TTF_GetError());
		getchar();
		return 1;
	} else if(!helpFont) {
		printf("TTF_OpenFont:\n%s\n\nPress [START] to exit.\n", TTF_GetError());
		getchar();
		return 1;
	}
	
	selectTime();
	countDown();
		
	quitApp();
	return 0;
}

void selectTime(void) {
	SDL_FillRect(screen, NULL, 0x0000);

	char topLine[256] = "[D-PAD] = Change time, [A] = Start";
	char bottomLine[256] = "[SELECT] = Quit";

	TTF_SizeText(helpFont, topLine, &width, &height);
	
	help1_surface = TTF_RenderText_Blended(helpFont, topLine, color);
	help2_surface = TTF_RenderText_Blended(helpFont, bottomLine, color);
	
	SDL_Rect dest;
	dest.x = 0;
	dest.y = height;
	dest.w = width;
	dest.h = height;
	
	SDL_BlitSurface(help1_surface, NULL, screen, NULL);
	SDL_BlitSurface(help2_surface, NULL, screen, &dest);
	
	int minutes = floor(countdownSecs / 60);
	int seconds = countdownSecs % 60;
	sprintf(countdownString, "%02d:%02d", minutes, seconds);
	TTF_SizeText(font, countdownString, &width, &height);
	
	left = ((320 - width) / 2);
	top = ((240 - height) / 2);
	
	SDL_Rect destination;
	destination.x = left;
	destination.y = top;
	destination.w = width;
	destination.h = height;
	
	text_surface = TTF_RenderText_Blended(font, countdownString, color);
	SDL_FillRect(screen, &destination, 0x0000); // Clear the screen before updating
	SDL_BlitSurface(text_surface, NULL, screen, &destination);
	SDL_Flip(screen);
	
	while(selectedTime == 0 && running == 1) {
		int minutes, seconds;
		SDL_Delay(10);
		
		SDL_Event event;
		while (SDL_PollEvent(&event)) {      
			if (event.type == SDL_QUIT) {
				running = 0;
			} else if(event.type == SDL_KEYDOWN) {
				switch(event.key.keysym.sym) {
					case DINGOO_BUTTON_SELECT: // Quit NAO
						running = 0;
						break;
					case DINGOO_BUTTON_UP: // Add a minute
						if(countdownSecs >= 5939) {
							countdownSecs = 5999;
						} else {
							countdownSecs += 60;
						}
						minutes = floor(countdownSecs / 60);
						seconds = countdownSecs % 60;
						sprintf(countdownString, "%02d:%02d", minutes, seconds);
						TTF_SizeText(font, countdownString, &width, &height);
						
						left = ((320 - width) / 2);
						top = ((240 - height) / 2);
						
						destination.x = left;
						destination.y = top;
						destination.w = width;
						destination.h = height;
						text_surface = TTF_RenderText_Blended(font, countdownString, color);
						SDL_FillRect(screen, &destination, 0x0000); // Clear the screen before updating
						SDL_BlitSurface(text_surface, NULL, screen, &destination);
						SDL_Flip(screen);
						break;
					case DINGOO_BUTTON_DOWN: // Subtract a minute
						if(countdownSecs < 61) {
							countdownSecs = 1;
						} else {
							countdownSecs -= 60;
						}
						minutes = floor(countdownSecs / 60);
						seconds = countdownSecs % 60;
						sprintf(countdownString, "%02d:%02d", minutes, seconds);
						TTF_SizeText(font, countdownString, &width, &height);
						
						left = ((320 - width) / 2);
						top = ((240 - height) / 2);
						
						destination.x = left;
						destination.y = top;
						destination.w = width;
						destination.h = height;
						text_surface = TTF_RenderText_Blended(font, countdownString, color);
						SDL_FillRect(screen, &destination, 0x0000); // Clear the screen before updating
						SDL_BlitSurface(text_surface, NULL, screen, &destination);
						SDL_Flip(screen);
						break;
					case DINGOO_BUTTON_LEFT: // Subtract a second
						if(countdownSecs > 1) {
							countdownSecs -= 1;
						} else {
							countdownSecs = 1;
						}
						minutes = floor(countdownSecs / 60);
						seconds = countdownSecs % 60;
						sprintf(countdownString, "%02d:%02d", minutes, seconds);
						TTF_SizeText(font, countdownString, &width, &height);
						
						left = ((320 - width) / 2);
						top = ((240 - height) / 2);
						
						destination.x = left;
						destination.y = top;
						destination.w = width;
						destination.h = height;
						text_surface = TTF_RenderText_Blended(font, countdownString, color);
						SDL_FillRect(screen, &destination, 0x0000); // Clear the screen before updating
						SDL_BlitSurface(text_surface, NULL, screen, &destination);
						SDL_Flip(screen);
						break;
					case DINGOO_BUTTON_RIGHT: // Add a second
						if(countdownSecs >= 5998) {
							countdownSecs == 5999;
						} else {
							countdownSecs += 1;
						}
						minutes = floor(countdownSecs / 60);
						seconds = countdownSecs % 60;
						sprintf(countdownString, "%02d:%02d", minutes, seconds);
						TTF_SizeText(font, countdownString, &width, &height);
						
						left = ((320 - width) / 2);
						top = ((240 - height) / 2);
						
						destination.x = left;
						destination.y = top;
						destination.w = width;
						destination.h = height;
						text_surface = TTF_RenderText_Blended(font, countdownString, color);
						SDL_FillRect(screen, &destination, 0x0000); // Clear the screen before updating
						SDL_BlitSurface(text_surface, NULL, screen, &destination);
						SDL_Flip(screen);
						break;
					case DINGOO_BUTTON_A: // START TIMER
						selectedTime = 1;
						countDown();
						break;
					default:
						break;
				}
			}
		}
	}
}

void countDown(void) {
	SDL_FillRect(screen, NULL, 0x0000);

	counting = 1;
	
	char topLine[256] = "[A] = Back to time select";
	char bottomLine[256] = "[SELECT] = Quit";

	TTF_SizeText(helpFont, topLine, &width, &height);
	
	help1_surface = TTF_RenderText_Blended(helpFont, topLine, color);
	help2_surface = TTF_RenderText_Blended(helpFont, bottomLine, color);
	
	SDL_Rect dest;
	dest.x = 0;
	dest.y = height;
	dest.w = width;
	dest.h = height;
	
	SDL_BlitSurface(help1_surface, NULL, screen, NULL);
	SDL_BlitSurface(help2_surface, NULL, screen, &dest);

	while(counting == 1 && running == 1) {
		if(countdownSecs == 0) {
			countdownSecs = 60;
			system("nohup madplay -Q alarm.mp3 &");
			selectedTime = 0;
			counting = 0;
			SDL_Delay(3000);
			system("killall madplay");
			selectTime();
		} else {
			countdownSecs--;
		}
	
		int minutes = floor(countdownSecs / 60);
		int seconds = countdownSecs % 60;
		sprintf(countdownString, "%02d:%02d", minutes, seconds);
		TTF_SizeText(font, countdownString, &width, &height);

		
		left = ((320 - width) / 2);
		top = ((240 - height) / 2);
		
		SDL_Rect destination;
		destination.x = left;
		destination.y = top;
		destination.w = width;
		destination.h = height;
		
		text_surface = TTF_RenderText_Blended(font, countdownString, color);
		
		SDL_FillRect(screen, &destination, 0x0000); // Clear the screen before updating
		SDL_BlitSurface(text_surface, NULL, screen, &destination);
		SDL_Flip(screen);
		
		SDL_Delay(500);
		
		SDL_Event event;
		while (SDL_PollEvent(&event)) {      
			if (event.type == SDL_QUIT) {
				running = 0;
			} else if(event.type == SDL_KEYDOWN) {
				switch(event.key.keysym.sym) {
					case DINGOO_BUTTON_SELECT:
						running = 0; // Quit NAO
						break;
					case DINGOO_BUTTON_A:
						counting = 0;
						selectedTime = 0;
						selectTime();
						break;
					default:
						break;
				}
			}
		}
		
		SDL_Delay(500);
	}
}

void quitApp(void) {
	// Close SDL_ttf
	SDL_FreeSurface(text_surface);
	SDL_FreeSurface(help1_surface);
	SDL_FreeSurface(help2_surface);
	TTF_CloseFont(font);
	TTF_CloseFont(helpFont);
	font = NULL;
	helpFont = NULL;
	TTF_Quit();
	
	// Close SDL
	SDL_FreeSurface(screen);
	screen = NULL;
	SDL_Quit();
}
